/* Copyright (C) 2005 E.Berta

netchoose is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation; either version 2, or (at your option) any later
version.

netchoose is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with GCC; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.
*/


#include <iostream>
#include <vector>
#include <fstream>

#include <string.h>


#include "supportfuncs.h"
#include "celldb_form.h"

using namespace std;

bool dbNeedSave = false;

Cell dbCell;
xmlNodePtr dbnode;

string Cell::touserstring() {
   
   string retstr = essid + string(" ") + address + string(" Encrypted:") + encrypted + string(" ") + bit_rate + string("Mb/s Quality:") + quality + string(" Ch:") + channel + string(" Mode:") + mode + string(" SigLevel:") + signal_level + string("dBm");
   
   return retstr;
}

void Cell::totable() const {
   extern celldb_form * w_db_ptr;
   
   dbNeedSave = true;
   
   if(this != &dbCell) {
      dbCell = *this;
   }

   QTable* table_ptr = w_db_ptr->required_table;
   table_ptr->setText(0,0, address);
   table_ptr->setText(1,0, essid);
   table_ptr->setText(2,0, interface);
   table_ptr->setText(3,0, key);

   table_ptr = w_db_ptr->manual_table;
   table_ptr->setText(0,0, ip);
   table_ptr->setText(1,0, netmask);
   table_ptr->setText(2,0, gateway);
   table_ptr->setText(3,0, nameserver);

   w_db_ptr->shellrun_edit->setText(sh_run);

   if(do_dhcp == string("yes")) {
      w_db_ptr->dhcp_checkbox->setChecked(true);
   } else
      w_db_ptr->dhcp_checkbox->setChecked(false);

   table_ptr->update();
}

void Cell::fromtable() {
   extern celldb_form * w_db_ptr;
   QTable* table_ptr = w_db_ptr->required_table;
   address = (table_ptr->text(0,0)).ascii();
   essid = (table_ptr->text(1,0)).ascii();
   interface = (table_ptr->text(2,0)).ascii();
   key = (table_ptr->text(3,0)).ascii();

   table_ptr = w_db_ptr->manual_table;
   ip = (table_ptr->text(0,0)).ascii();
   netmask = (table_ptr->text(1,0)).ascii();
   gateway = (table_ptr->text(2,0)).ascii();
   nameserver = (table_ptr->text(3,0)).ascii();

   if(w_db_ptr->dhcp_checkbox->isChecked())
      do_dhcp = "yes";
   else
      do_dhcp = string();

   sh_run = w_db_ptr->shellrun_edit->text().ascii();
}

void strsetenv(string envvar, string value) {
   if(value == string()) {
      unsetenv(envvar.c_str());
   } else {
      setenv(envvar.c_str(), value.c_str(), 1);
   }
}

int Cell::activate() {
   
   strsetenv("interface", interface);
   strsetenv("address", address);
   strsetenv("essid", essid);
   strsetenv("mode", mode);
   strsetenv("channel", channel);
   strsetenv("encrypted", encrypted);
   strsetenv("bit_rate", bit_rate);
   strsetenv("quality", quality);
   strsetenv("signal_level", signal_level);

   strsetenv("key", key);
   strsetenv("do_dhcp", do_dhcp);
   strsetenv("ip", ip);
   strsetenv("netmask", netmask);
   strsetenv("gateway", gateway);
   strsetenv("nameserver", nameserver);
  
   int shret = system(sh_run.c_str());

   fprintf(stderr, "shret: %d\n", shret);

   return shret;
    
}

xmlDocPtr doc;
void Cell::populatefrom(xmlNodePtr node, bool clearcur) {
   xmlChar* uri;

   if(clearcur) {
      *this = Cell();
      do_dhcp = string();
   }

   xmlnode = node;

   if((uri = xmlGetProp(node, (const xmlChar*)"interface"))) {
      this->interface = (char*)uri;
   }
   if((uri = xmlGetProp(node, (const xmlChar*)"address"))) {
      this->address = (char*)uri;
   }
   if((uri = xmlGetProp(node, (const xmlChar*)"essid"))) {
      this->essid = (char*)uri;
   }
   if((uri = xmlGetProp(node, (const xmlChar*)"channel"))) {
      this->channel = (char*)uri;
   }
   if((uri = xmlGetProp(node, (const xmlChar*)"encrypted"))) {
      this->encrypted = (char*)uri;
   }
   if((uri = xmlGetProp(node, (const xmlChar*)"key"))) {
      this->key = (char*)uri;
   }
   if((uri = xmlGetProp(node, (const xmlChar*)"do_dhcp"))) {
      this->do_dhcp = (char*)uri;
   }
   if((uri = xmlGetProp(node, (const xmlChar*)"ip"))) {
      this->ip = (char*)uri;
   }
   if((uri = xmlGetProp(node, (const xmlChar*)"netmask"))) {
      this->netmask = (char*)uri;
   }
   if((uri = xmlGetProp(node, (const xmlChar*)"gateway"))) {
      this->gateway = (char*)uri;
   }
   if((uri = xmlGetProp(node, (const xmlChar*)"nameserver"))) {
      this->nameserver = (char*)uri;
   }
   
   if((uri = xmlNodeListGetString(doc, node->xmlChildrenNode->xmlChildrenNode, 1))) {
      this->sh_run = (char*)uri;
   }
}

void Cell::setnode(xmlNodePtr cellnode) const {

   xmlAttrPtr attr = cellnode->properties;
   while(attr) {
      xmlAttrPtr prev_attr = attr;
      attr = attr->next;
      xmlRemoveProp(prev_attr);
   }
   xmlNodePtr children = cellnode->children;
   while(children) {
      xmlNodePtr rmnode = children;
      children = children->next;
      xmlUnlinkNode(rmnode);
      xmlFreeNodeList(rmnode);
   }
   if(interface != string())
      xmlSetProp(cellnode, (const xmlChar*)"interface", (const xmlChar*)interface.c_str());
   if(address != string())
      xmlSetProp(cellnode, (const xmlChar*)"address", (const xmlChar*)address.c_str());
   if(essid != string())
      xmlSetProp(cellnode, (const xmlChar*)"essid", (const xmlChar*)essid.c_str());
   if(channel != string())
      xmlSetProp(cellnode, (const xmlChar*)"channel", (const xmlChar*)channel.c_str());
   if(encrypted != string())
      xmlSetProp(cellnode, (const xmlChar*)"encrypted", (const xmlChar*)encrypted.c_str());
   if(key != string())
      xmlSetProp(cellnode, (const xmlChar*)"key", (const xmlChar*)key.c_str());
   if(do_dhcp != string())
      xmlSetProp(cellnode, (const xmlChar*)"do_dhcp", (const xmlChar*)do_dhcp.c_str());
   if(ip != string())
      xmlSetProp(cellnode, (const xmlChar*)"ip", (const xmlChar*)ip.c_str());
   if(netmask != string())
      xmlSetProp(cellnode, (const xmlChar*)"netmask", (const xmlChar*)netmask.c_str());
   if(gateway != string())
      xmlSetProp(cellnode, (const xmlChar*)"gateway", (const xmlChar*)gateway.c_str());
   if(nameserver != string())
      xmlSetProp(cellnode, (const xmlChar*)"nameserver", (const xmlChar*)nameserver.c_str());
   
   xmlNewChild(cellnode, NULL, (const xmlChar*)"SH_RUN", (const xmlChar*)sh_run.c_str());
   
}

void dbRead() {
   doc = xmlParseFile("/usr/local/share/netchoose/cells.xml");
   if (doc == NULL ) {
      fprintf(stderr,"cells.xml not parsed successfully. \n");
   }
}

void dbWrite() {
   xmlSaveFormatFile("/usr/local/share/netchoose/cells.xml", doc, 1);
   dbNeedSave = false;
}

xmlNodePtr getNextElement(xmlNodePtr cur) {
   cur = cur->next;
   while(cur != NULL) {
      if(cur->type != XML_ELEMENT_NODE) {
         cur = cur->next;
         continue;
      }
      return cur;
   }
   return cur;
}

xmlNodePtr getPrevElement(xmlNodePtr cur) {
   cur = cur->prev;
   while(cur != NULL) {
      if(cur->type != XML_ELEMENT_NODE) {
         cur = cur->prev;
         continue;
      }
      return cur;
   }
   return cur;
}

extern celldb_form* w_db_ptr;
void dbCellUpdate() {
   if(dbCell.xmlnode) {
      w_db_ptr->required_table->setReadOnly(true);
      w_db_ptr->manual_table->setReadOnly(true);
      dbCell.fromtable();
      dbCell.setnode(dbCell.xmlnode);
      w_db_ptr->required_table->setReadOnly(false);
      w_db_ptr->manual_table->setReadOnly(false);
   }
}

void dbDisplay(xmlNodePtr cellnode) {
   
   dbCellUpdate();

   dbCell.populatefrom(cellnode);
   dbCell.totable();
}

xmlNodePtr dbGetFirst() {
   xmlNodePtr cur = xmlDocGetRootElement(doc);
   cur = cur->xmlChildrenNode;
   while(cur != NULL) {
      if(cur->type != XML_ELEMENT_NODE) {
         cur = cur->next;
         continue;
      }
      return cur;
   }
   return cur;
}

xmlNodePtr dbExists(bool& found, const Cell& cell) {
   xmlNodePtr cur, rootnode;
   xmlNodePtr defaultcur = NULL;

   found = false;

   rootnode = xmlDocGetRootElement(doc);
   cur = rootnode; 
   if (cur == NULL) {
      fprintf(stderr,"empty document\n");
      return NULL;
   }
   
   if (xmlStrcmp(cur->name, (const xmlChar *) "KNOWNCELLS")) {
      fprintf(stderr,"Unknown root element in cells.xml");
      return NULL;
   }

   cur = cur->xmlChildrenNode;
   while(cur != NULL) {
      if(cur->type != XML_ELEMENT_NODE) {
         cur = cur->next;
         continue;
      }
      
      {
         xmlChar *uri;
         uri = xmlGetProp(cur, (const xmlChar*)"address");
         if(!uri) {
            cur = cur->next;
            xmlFree(uri);
            continue;
         }
         if(!xmlStrcmp(uri, (const xmlChar*)cell.address.c_str())) {
            cout << "Match!\n";
            found = true;
            xmlFree(uri);
            return cur;
         } else if(!xmlStrcmp(uri, (const xmlChar*)"default")) {
            defaultcur = cur;
         }
         
         xmlFree(uri);
      }
    
      cur = cur->next;
   }
   
   return defaultcur;
}

void dbAdd(Cell& cell) {
   xmlNodePtr newnode;
   newnode = xmlNewChild(xmlDocGetRootElement(doc), NULL, (const xmlChar*)"CELL", NULL);
   cell.setnode(newnode);
   cell.xmlnode = newnode;
}

void dbRemove(xmlNodePtr node) {
   if(node == NULL)
      node = dbCell.xmlnode;
   if(node == NULL)
      fprintf(stderr, "Bad dbCell xmlnode\n");
   xmlUnlinkNode(node);
   xmlFreeNodeList(node);
   dbCell.xmlnode = NULL;
}


std::vector<std::string> strbreak(const std::string& foo, char bchar = '\n') {
   std::vector<std::string> lines;

   std::string::size_type pos, next_pos;
  
   pos = foo.find_first_not_of(bchar, 0);

   const std::string::size_type endpos = foo.size();

   while((pos < std::string::npos) &&
         (pos < endpos)) {
    
      next_pos = foo.find(bchar, pos);

      lines.push_back(foo.substr(pos, next_pos-pos));

      //pos = next_pos+1;
      pos = foo.find_first_not_of(bchar, next_pos);

   }
  
   return lines;
}


string getvalue(const std::string& foo, char bchar = ':') {
   vector<string> sbreaks = strbreak(foo, bchar);
   if(sbreaks.size() != 2) return string();

   return sbreaks[1];
}

string cutparen(std::string foo) {
   if(foo.length() <= 2) return foo;
   if(foo.at(0) == '\"') foo.erase(0, 1);
   if(foo.at(foo.length()-1) == '\"') foo.erase(foo.length()-1, 1);

   return foo;
}

bool sncmp(const std::string& foo, const char * cmp_cstr, int num = -1) {
   if(num == -1)  num = strlen(cmp_cstr);
   return(strncmp(foo.c_str(), cmp_cstr, num)==0);

}

string cur_interface;

Cell cur_cell;

vector<Cell> cells;

int refresh_cells(QListBox* celllist_box) {

   string iwlist_in;
   char tempname[32];
   char iwrun[64];

   cells.clear();
   celllist_box->clear();

   sprintf(tempname, "/tmp/netchoose_iwlist.%d", getpid());

   sprintf(iwrun, "iwlist scan > %s 2> /dev/null", tempname);

   //if(system("cat iwsamp.txt > /tmp/nc_iwlist 2> /dev/null") != 0) {
   if(system(iwrun) != 0) {
      fprintf(stderr, "Could not find iwlist. Perhaps not in your PATH.\n");
      return -1;
   }

   ifstream in(tempname);	

   if(!in) {
      return -2;
   }

   {
      string line; 

      while(getline(in,line))		
      {
         iwlist_in += line + "\n"; 
      }

      in.close(); 
	
   }
   
   sprintf(iwrun, "rm %s", tempname);
   system(iwrun);

   vector<string> iwlines = strbreak(iwlist_in);
   for(vector<string>::iterator siter = iwlines.begin(); siter != iwlines.end();
       siter++) {

      vector<string> iwsplit = strbreak(*siter, ' ');

      if(iwsplit.size() < 1) continue;

      if(iwsplit.size() >= 3)
         if((iwsplit[1] == string("Scan")) && (iwsplit[2] == string("completed"))) {
            if(cur_cell.address != string()) {
               cells.push_back(cur_cell);
               cur_cell.address = string();
            }
            //cout << "Interface: " << iwsplit[0] << "\n"; 
            cur_interface =  iwsplit[0];
         }

      if(iwsplit.size() >= 5)
         if((iwsplit[0] == string("Cell")) && (iwsplit[3] == string("Address:"))) {
            if(cur_cell.address != string()) {
               cells.push_back(cur_cell);
               cur_cell.address = string();
            }
            //cout << "Address: " << iwsplit[4] << "\n";
            cur_cell.interface = cur_interface; 
            cur_cell.address = iwsplit[4];
         }
  
      if(iwsplit.size() >= 1)
         if(sncmp(iwsplit[0], "ESSID:")) {
            cur_cell.essid = cutparen(getvalue(iwsplit[0]));
            //cout << "essid: " << cur_cell.essid << "\n";
         } 
  
      if(iwsplit.size() >= 1)
         if(sncmp(iwsplit[0], "Mode:")) {
            cur_cell.mode = cutparen(getvalue(iwsplit[0]));
            //cout << "essid: " << cur_cell.essid << "\n";
         } 
  
  
      if(iwsplit.size() >= 1)
         if(sncmp(iwsplit[0], "Channel:")) {
            cur_cell.channel = getvalue(iwsplit[0]);
            //cout << "channel: " << cur_cell.channel << "\n";
         }

      if(iwsplit.size() >= 2)
         if(sncmp(iwsplit[0], "Encryption")) {
            cur_cell.encrypted = getvalue(iwsplit[1]);
            if(strncasecmp(cur_cell.encrypted.c_str(), "on", 2) == 0) {
               cur_cell.encrypted = "yes";
            } else
               cur_cell.encrypted = "no";
            //cout << "encrypted: " << cur_cell.encrypted << "\n";
         }

      if(iwsplit.size() >= 3)
         if(sncmp(iwsplit[0], "Bit")) {
            cur_cell.bit_rate = getvalue(iwsplit[1]);
         }

      if(iwsplit.size() >= 4)
         if(sncmp(iwsplit[0], "Quality")) {
            cur_cell.quality = getvalue(iwsplit[0],'=');
            cur_cell.signal_level = getvalue(iwsplit[2], '=');
         }


   }

   if(cur_cell.address != string()) {
      cells.push_back(cur_cell);
      cur_cell.address = string();
   }

   cout << "numcells: " << cells.size() << "\n";

   
   for(vector<Cell>::iterator citer = cells.begin(); citer != cells.end(); citer++) {

      //cout << citer->touserstring() << "\n";
      celllist_box->insertItem(citer->touserstring()); 

   }
   if(cells.size()) celllist_box->setCurrentItem(0);  

   return 0;
}
 
